# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class TronVote(p.MessageType):

    def __init__(
        self,
        vote_address: bytes = None,
        vote_count: int = None,
    ) -> None:
        self.vote_address = vote_address
        self.vote_count = vote_count

    @classmethod
    def get_fields(cls):
        return {
            1: ('vote_address', p.BytesType, 0),
            2: ('vote_count', p.UVarintType, 0),
        }
