package org.ethereum.lists.tokens.model

data class Logo(val src: String,
                val width: String? = null,
                val height: String? = null,
                val ipfs_hash: String? = null)