# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class TezosContractID(p.MessageType):

    def __init__(
        self,
        tag: int = None,
        hash: bytes = None,
    ) -> None:
        self.tag = tag
        self.hash = hash

    @classmethod
    def get_fields(cls):
        return {
            1: ('tag', p.UVarintType, 0),
            2: ('hash', p.BytesType, 0),
        }
