# Automatically generated by pb2py
# fmt: off
from .. import protobuf as p


class OntologyOntIdAttribute(p.MessageType):

    def __init__(
        self,
        key: str = None,
        type: str = None,
        value: str = None,
    ) -> None:
        self.key = key
        self.type = type
        self.value = value

    @classmethod
    def get_fields(cls):
        return {
            1: ('key', p.UnicodeType, 0),
            2: ('type', p.UnicodeType, 0),
            3: ('value', p.UnicodeType, 0),
        }
