package org.ethereum.lists.tokens.model

data class Social(
        val blog: String? = null,
        val chat: String? = null,
        val facebook: String? = null,
        val forum: String? = null,
        val discord: String? = null,
        val github: String? = null,
        val gitter: String? = null,
        val instagram: String? = null,
        val linkedin: String? = null,
        val reddit: String? = null,
        val slack: String? = null,
        val telegram: String? = null,
        val twitter: String? = null,
        val medium: String? = null,
        val bitcointalk: String? = null,
        val googleplus: String? = null,
        val vimeo: String? = null,
        val youtube: String? = null)